#!/bin/bash
# Live OpenCloudOS Backup, 2020
# Songqiao Tao <joeytao@tencent.com>

new_dir()
{
    local newdir="$*"
    i=0
    while [ -e $newdir ]; do
        i=`expr $i + 1`
        newdir="$*-$i"
    done
    echo $newdir
}

rebuildtree()
{ 
# Remounting the linux directories effectively excludes removable media, manually mounted devices, windows partitions, virtual files under /proc, /sys, /dev, etc. If your partition scheme is more complicated than listed below, you must add lines to rebuildtree() and destroytree(), otherwise the backup will be partial.
    mkdir /$1
    mount --make-private --bind / /$1
    mount --make-private --bind /boot /$1/boot
    mount --make-private --bind /boot/efi /$1/boot/efi
    mount --make-private --bind /home /$1/home
    #mount --make-private --bind /tmp /$1/tmp
    #mount --make-private --bind /usr /$1/usr
    #mount --make-private --bind /var /$1/var
    #mount --make-private --bind /srv /$1/srv
    #mount --make-private --bind /opt /$1/opt
    mount --make-private --bind /usr/local /$1/usr/local
}

destroytree()
{
    umount /$1/usr/local
    #umount /$1/opt
    #umount /$1/srv
    #umount /$1/var
    #umount /$1/usr
    #umount /$1/tmp
    umount /$1/home
    umount /$1/boot/efi
    umount /$1/boot
    umount /$1
    rmdir /$1
}

ocos_backup()
{
    if [ -n "$1" ]; then
        back_op=$1
	if [ "$back_op"x == "reboot"x ];then
            /usr/lib/opencloudos-tools/opencloudos_super_tool.py -b
	else
	    echo "Please type the correct parameters!"
	    echo "ocos -b : backup the system online"
	    echo "ocos -b reboot : reboot to backup the system"
	fi
	exit 0
    fi
    if [ ! -x /usr/sbin/mksquashfs ]; then
        echo "Try to install squashfs-tools rpm by yum!"
        yum -y install squashfs-tools > /dev/null 2>&1
    fi
    if [ ! -x /usr/sbin/mksquashfs ]; then
        echo "/usr/sbin/mksquashfs not found, please install squashfs-tools first!"
        exit -1
    fi

    bindingdir=`new_dir /tmp/bind`
    backupdir="/data/opencloudos/backup"
    bindingdir="${bindingdir#/}"
    backupdir="${backupdir#/}"
    
    exclude=`new_dir /tmp/exclude`
    echo $backupdir > $exclude
    echo $bindingdir >> $exclude
    echo etc/udev/rules.d/70-persistent-net.rules >> $exclude
    echo etc/machine-id >> $exclude
    echo lost+found >> $exclude
    echo data/lost+found >> $exclude
    echo usr/local/lost+found >> $exclude
    echo var/cache/yum >> $exclude

    for i in `swapon -s | grep file | cut -d " " -f 1`; do
        echo "${i#/}" >> $exclude
    done
    
    for i in `ls /tmp -A`; do
        echo "tmp/$i" >> $exclude
    done

    rebuildtree $bindingdir

    VER=$(awk '/OpenCloudOS/{print $(NF)}' /etc/opencloudos-release)
    mkdir -p "/$backupdir"
    sqfs=opencloudos-64bit-v${VER}-backup.`date +"%Y-%m-%d-%H:%M"`.sqfs
    mksquashfs /$bindingdir "/$backupdir/${sqfs}" -comp xz -b 262144 -ef $exclude
    destroytree $bindingdir
    rm $exclude
    echo "Your backup is ready in /$backupdir/${sqfs}"
}

#ocos_backup
